project_dir = File.expand_path("..", File.dirname(__FILE__))
require File.join(project_dir, 'custom/versions')
require File.join(project_dir, 'checks/edp_connection_check')
require File.join(project_dir, 'checks/edp_client_dir_check')
require File.join(project_dir, 'checks/edp_authorization_check')
require File.join(project_dir, 'checks/webuser_exists_check')
require File.join(project_dir, 'checks/webuser_url_check')
require File.join(project_dir, 'checks/webuser_api_key_check')

module AbasBpm
  class CheckFactory

    def initialize(data)
      @config = data['config']
      @registry_credentials = Abas::Credentials.load(@config['authentication']['registry_credentials'])    
      @database_credentials = Abas::Credentials.load(@config['authentication']['database_credentials'])    
    end

    def engine_port_free_check()
      Abas::PortCheck.create(name: "Check if abas BPM engine port is free", port: @config['setup']['engine_port'], free: true)
    end

    def engine_port_used_check()
      Abas::PortCheck.create(name: "Check if abas BPM engine port is used", port: @config['setup']['engine_port'], free: false)
    end

    def db_port_free_check()
      Abas::PortCheck.create(name: "Check if abas BPM db port is free", port: @config['setup']['database_port'], free: true)
    end

    def db_port_used_check()
      Abas::PortCheck.create(name: "Check if abas BPM db port is used", port: @config['setup']['database_port'], free: false)
    end

    def install_dir_exists_check()
      Abas::FileCheck.create(name: "Check if install_dir exists", path: @config['install_dir'], should_exist: true)
    end

    def install_dir_not_exists_check()
      Abas::FileCheck.create(name: "Check if install_dir does not exist", path: @config['install_dir'], should_exist: false)
    end

    def disk_space_check()
      Abas::DiskCheck.create(
        name: "Check if install_dir has enough free space",
        path: @config['install_dir'],
        required_mb: 1024 * 2,
        recommended_mb: 1024 * 10
      )
    end

    def credential_check()
      Abas::CredentialCheck.create(
        name: "Check if credentials are valid",
        username: @registry_credentials['username'],
        password: @registry_credentials['password'],
      )
    end

    def user_check()
      Abas::UserCheck.create(
        name: "Check if system user exists",
        user: @config['user'] 
      )
    end

    def engine_artifact_check()
      Abas::ArtifactCheck.create(
        name: "Check if abas BPM Engine is accessible",
        username: @registry_credentials['username'],
        password: @registry_credentials['password'],
        artifact: "docker/abas-bpm-engine/#{engine_version()}/"
      )
    end

    def engine_running_check()
      Abas::DockerContainerRunningCheck.create(
        name: "Check if abas BPM engine is running",
        container_name: "#{@config['setup']['project_name']}-engine"
      )
    end

    def db_running_check()
      Abas::DockerContainerRunningCheck.create(
        name: "Check if abas BPM db is running",
        container_name: "#{@config['setup']['project_name']}-db"
      )
    end

    def edp_configuration_checks
      checks = Array.new

      edp_configs = @config['configuration']['edp_configs']
      
      edp_configs.each do |edp_config|
        edp_host_ip = edp_config['edp']['ip']
        edp_host_port = edp_config['edp']['port']
        client_dir = edp_config['client']['clientdir']
        credentials = Abas::Credentials.load(edp_config['credentials'])
        password = credentials['password']

        edp_credential_check = AbasBpm::EdpConnectionCheck.create(
          name: "Check if a edp connection can be established",
          ip: edp_host_ip,
          port: edp_host_port,
          container_name: "#{@config['setup']['project_name']}-engine"
        )

        edp_client_dir_check = AbasBpm::EdpClientDirCheck.create(
          name: "Check if client (#{client_dir}) is accessible with edp",
          ip: edp_host_ip,
          port: edp_host_port,
          client_dir: client_dir,
          container_name: "#{@config['setup']['project_name']}-engine"
        )

        edp_authorization_check = AbasBpm::EdpAuthorizationCheck.create(
          name: "Check if client (#{client_dir}) is accessible with edp and given password",
          ip: edp_host_ip,
          port: edp_host_port,
          client_dir: client_dir,
          password: password,
          container_name: "#{@config['setup']['project_name']}-engine"
        )

        webuser_exists_check = AbasBpm::WebuserExistsCheck.create(
          name: "Check if client (#{client_dir}) has the abas-bpm webuser setup",
          ip: edp_host_ip,
          port: edp_host_port,
          client_dir: client_dir,
          password: password,
          container_name: "#{@config['setup']['project_name']}-engine"
        )

        webuser_url_check = AbasBpm::WebuserUrlCheck.create(
          name: "Check if client (#{client_dir}) webuser backend url is valid",
          ip: edp_host_ip,
          port: edp_host_port,
          client_dir: client_dir,
          password: password,
          container_name: "#{@config['setup']['project_name']}-engine",
          engine_url: "http://#{`hostname -i`.strip}:#{@config['setup']['engine_port']}/engine-rest"
        )

        webuser_api_key_check = AbasBpm::WebuserApiKeyCheck.create(
          name: "Check if client (#{client_dir}) webuser api key is valid",
          ip: edp_host_ip,
          port: edp_host_port,
          client_dir: client_dir,
          password: password,
          container_name: "#{@config['setup']['project_name']}-engine",
          database_container_name: "#{@config['setup']['project_name']}-db",
          database_user_name: "#{@database_credentials['username']}"
        )

        checks << (edp_credential_check)
        checks << (edp_client_dir_check)
        checks << (edp_authorization_check)
        checks << (webuser_exists_check)
        checks << (webuser_url_check)
        checks << (webuser_api_key_check)

        edp_client_dir_check.add_condition(edp_credential_check)
        edp_authorization_check.add_condition(edp_client_dir_check)
        webuser_exists_check.add_condition(edp_authorization_check)
        
        webuser_url_check.add_condition(webuser_exists_check)
        webuser_api_key_check.add_condition(webuser_exists_check)

        return checks
      end 

    end

    private 

    def engine_version
      return AbasBpm::Versions.engine_version(version: @config['setup']['version']) unless @config['developer_settings']['engine_version']
      return @config['developer_settings']['engine_version']
    end
 
  end
end