module AbasBpm
    require File.join(File.dirname(__FILE__), 'custom/versions')

    class Extras
        def initialize(module_data, credentials_handler)
            @credentials_handler    = credentials_handler
            @resolved_config        = module_data['resolved_config']
            @host                   = module_data['host']
            @install_dir            = @resolved_config['install_dir']
            @setup                  = @resolved_config['setup']
            @authentication         = @resolved_config['authentication']
            @database_credentials   = @authentication['database_credentials']
            @license                = @resolved_config['license']
            @configuration          = @resolved_config['configuration']
            @smtp_configuration     = @configuration['smtp']
            @developer_settings     = @resolved_config['developer_settings']
        end

        def services_definition
            h = Hash.new
            h['version'] = '3'
            h['services'] = Hash.new
            h['services']['postgresql'] = Hash.new
            h['services']['postgresql']['image'] = 'postgres:9.6'
            h['services']['postgresql']['container_name'] = "#{@setup['project_name']}-db"
            h['services']['postgresql']['environment'] = Array.new
            h['services']['postgresql']['environment'].push("POSTGRES_USER=#{@credentials_handler.username(@database_credentials)}")
            h['services']['postgresql']['environment'].push("POSTGRES_PASSWORD=#{@credentials_handler.password(@database_credentials)}")
            h['services']['postgresql']['environment'].push("POSTGRES_DB=camunda")
            h['services']['postgresql']['environment'].push("PSQL_TRUST_LOCALNET=true")

            h['services']['postgresql']['volumes'] = Array.new
            h['services']['postgresql']['volumes'].push("#{@install_dir}/data:/var/lib/postgresql/data")

            if @setup['database_port'] && @setup['database_port'] != ""
                h['services']['postgresql']['ports'] = Array.new 
                h['services']['postgresql']['ports'].push("#{@setup['database_port']}:5432")
            end

            h['services']['camunda'] = Hash.new
            h['services']['camunda']['image'] = "docker.registry.abas.sh/abas-bpm-engine:#{engine_version()}"
            h['services']['camunda']['container_name'] = "#{@setup['project_name']}-engine"
            h['services']['camunda']['environment'] = Array.new
            h['services']['camunda']['environment'].push("DB_DRIVER=postgresql")
            h['services']['camunda']['environment'].push("DB_URL=jdbc:postgresql://db:5432/camunda")
            h['services']['camunda']['environment'].push("DB_USERNAME=#{@credentials_handler.username(@database_credentials)}")
            h['services']['camunda']['environment'].push("DB_PASSWORD=#{@credentials_handler.password(@database_credentials)}")

            unless @smtp_configuration.nil?
                h['services']['camunda']['environment'].push("SMTP_HOST=#{@smtp_configuration['host']}")
                h['services']['camunda']['environment'].push("SMTP_PORT=#{@smtp_configuration['port']}")
                h['services']['camunda']['environment'].push("SMTP_USER=#{@smtp_configuration['user']}")
                h['services']['camunda']['environment'].push("SMTP_PASSWORD=#{@smtp_configuration['password']}")
                h['services']['camunda']['environment'].push("SMTP_TLS_ENABLED=#{@smtp_configuration['tls_enabled']}")
                h['services']['camunda']['environment'].push("SMTP_AUTH=#{@smtp_configuration['auth']}")
                h['services']['camunda']['environment'].push("SMTP_FROM=#{@smtp_configuration['from']}")
            end

            h['services']['camunda']['environment'].push("TZ=#{@configuration['timezone']}")

            unless business_hours_configuration().nil?
                h['services']['camunda']['environment'].push("BUSINESSHOURS_CONFIG=#{business_hours_configuration().to_json}")
            end

            h['services']['camunda']['environment'].push("UI_CONFIG=#{ui_configuration().to_json}")
            h['services']['camunda']['environment'].push("EDP_CONFIG=#{edp_configuration().to_json}")
            h['services']['camunda']['environment'].push("EDP_LOG_ENABLED=#{@configuration['edp_log_enabled']}")
            h['services']['camunda']['environment'].push("EDP_LOCK_WAIT_DURATION=#{@configuration['edp_lock_wait_duration']}")
            h['services']['camunda']['environment'].push("EDP_END_SESSION_MODE=#{@configuration['edp_end_session_mode']}")

            if @license['license_controller'] && @license['license_controller']['license_controller_url']
                h['services']['camunda']['environment'].push("LICENSE_CONTROLLER_URL=#{@license['license_controller']['license_controller_url']}")
            end

            if @developer_settings && @developer_settings['license_controller_url']
                h['services']['camunda']['environment'].push("LICENSE_CONTROLLER_URL2=#{@developer_settings['license_controller_url']}")
            end

            if @setup['extra_hosts'] && @setup['extra_hosts'].length > 0
                h['services']['camunda']['extra_hosts'] = Array.new
                @setup['extra_hosts'].each do |val|
                    h['services']['camunda']['extra_hosts'].push("#{val['hostname']}:#{val['ip']}")
                end
            end

            h['services']['camunda']['volumes'] = Array.new
            h['services']['camunda']['volumes'].push("#{@install_dir}/oauth2.properties:/camunda/standalone/configuration/oauth2.properties")
        
            h['services']['camunda']['depends_on'] = Array.new
            h['services']['camunda']['depends_on'].push("postgresql")
        
            h['services']['camunda']['links'] = Array.new
            h['services']['camunda']['links'].push("postgresql:db")

            h['services']['camunda']['ports'] = Array.new
            h['services']['camunda']['ports'].push("#{@setup['engine_port']}:8080")
            
            h['services']['camunda']['logging'] = Hash.new
            h['services']['camunda']['logging']['driver'] = "json-file"
            h['services']['camunda']['logging']['options'] = Hash.new
            h['services']['camunda']['logging']['options']['max-size'] = "10m"
            h['services']['camunda']['logging']['options']['max-file'] = "50"

            return h
        end

        def engine_version
            return AbasBpm::Versions.engine_version(version: @resolved_config['setup']['version']) unless @resolved_config['developer_settings']['engine_version']
            return @resolved_config['developer_settings']['engine_version']
        end

        def is_business_hours_set
            business_hours = @configuration['business_hours']

            return business_hours['monday'].nil? && 
            business_hours['tuesday'].nil? &&
            business_hours['wednesday'].nil? &&
            business_hours['thursday'].nil? &&
            business_hours['friday'].nil? &&
            business_hours['saturday'].nil? &&
            business_hours['sunday'].nil?

        end
        
        def business_hours_configuration
            business_hours = @configuration['business_hours']

            if is_business_hours_set()
                return nil
            end
            
            business_hours_configuration = Hash.new
            business_hours_configuration[:version] = '1.0'
            business_hours_configuration[:Mon] = business_hours['monday']
            business_hours_configuration[:Tue] = business_hours['tuesday']
            business_hours_configuration[:Wed] = business_hours['wednesday']
            business_hours_configuration[:Thu] = business_hours['thursday']
            business_hours_configuration[:Fri] = business_hours['friday']
            business_hours_configuration[:Sat] = business_hours['saturday']
            business_hours_configuration[:Sun] = business_hours['sunday']

            # Remove all weekday keys with nil value
            business_hours_configuration.reject!{|k,v| v.nil?}

            return business_hours_configuration
        end

        def ui_configuration
            options = Array.new
            options.push("eks") if @configuration['ui_options']['abas_gui_embedded']
            options.push("2016") if @configuration['ui_options']['abas_gui_2016']
            options.push("2017") if @configuration['ui_options']['abas_gui']
            options.push("webui") if @configuration['ui_options']['webui']
            options.push("webclient") if @configuration['ui_options']['webclient']
            return options
        end

        def edp_configuration
            return Array.new unless @configuration['edp_configs']
            configuration = Array.new

            edp_configs = @configuration['edp_configs']
            edp_configs.each_with_index do |obj, index| 
                edpConfig = Hash.new
                edpConfig[:id] = obj['client']['mandant']
                edpConfig[:client] = obj['client']['clientdir']
                edpConfig[:displayName] = obj['display_name']
                edpConfig[:defaultConfig] = index == 0
                edpConfig[:host] = obj['edp']['ip']
                edpConfig[:port] = obj['edp']['port']
                edpConfig[:user] = @credentials_handler.username(obj['credentials'])
                edpConfig[:user] ||= ''
                edpConfig[:password] = @credentials_handler.password(obj['credentials'])
                edpConfig[:password] ||= ''
                edpConfig[:ddeServer] = obj['dde_server_name']
                edpConfig[:restApiId] = obj['rest_api_id']

                configuration.push(edpConfig)
            end

            return configuration
        end
    end
end