# abas_bpm.rb
require 'yaml' # import

module AbasBpm

  project_dir = File.expand_path("..", File.dirname(__FILE__))
  require File.join(project_dir, 'custom/versions')

  class Main
    def initialize(identifier_hash)
        @install_dir = identifier_hash['install_dir']
        @docker_compose_file_path = File.join(@install_dir, "/docker-compose.yml")

        if exists?
            @dc_file = DockerUtil.loadComposeFile(@docker_compose_file_path)
        end
    end

    def exists?
        return File.file?(@docker_compose_file_path)
    end

    def user
        `stat -c "%U" #{@install_dir}`.strip
    end

    def setup
        obj = Hash.new
        obj[:project_name] = `cat #{@install_dir}/.env`.strip.split("=").last
        obj[:version] = AbasBpm::Versions.find_version(engine_version: @dc_file['services']['camunda']['image'].split(":").last)
        obj[:engine_port] = @dc_file['services']['camunda']['ports'][0].split(":")[0]
        obj[:database_port] = @dc_file['services']['postgresql']['ports'].select{|env|env.end_with?(":5432")}.first.split(":").first.strip
        return obj
    end

    def authentication
        obj = Hash.new

        oauth2_properties = Properties.new("#{@install_dir}/oauth2.properties")

        oauth2 = Hash.new
        oauth2[:issuer] = oauth2_properties['issuer']
        oauth2[:jwks_url] = oauth2_properties['jwksurl']
        oauth2[:user_info_endpoint] = oauth2_properties['userInfoEndpoint']
        oauth2[:audience] = oauth2_properties['audience']
        obj[:oauth2] = oauth2

        return obj
    end

    def configuration
        obj = Hash.new

        oauth2_properties = Properties.new("#{@install_dir}/oauth2.properties")
        obj[:admin_user_email] = oauth2_properties['adminMail']

        obj[:timezone] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'TZ')

        smtp_params = Hash.new
        smtp_params[:host] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'SMTP_HOST')
        smtp_params[:port] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'SMTP_PORT')
        smtp_params[:user] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'SMTP_USER')
        smtp_params[:password] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'SMTP_PASSWORD')
        smtp_params[:tls_enabled] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'SMTP_TLS_ENABLED')
        smtp_params[:auth] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'SMTP_AUTH')
        smtp_params[:from] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'SMTP_FROM')
        smtp_params.reject!{|k,v| v.nil?}
        obj[:smtp] = smtp_params

        business_hours_config = Hash.new
        @dc_bhc_string = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'BUSINESSHOURS_CONFIG')
        if @dc_bhc_string
            dc_bhc = JSON.parse(@dc_bhc_string)
            business_hours_config[:monday] = dc_bhc['Mon']
            business_hours_config[:tuesday] = dc_bhc['Tue']
            business_hours_config[:wednesday] = dc_bhc['Wed']
            business_hours_config[:thursday] = dc_bhc['Thu']
            business_hours_config[:friday] = dc_bhc['Fri']
            business_hours_config[:saturday] = dc_bhc['Sat']
            business_hours_config[:sunday] = dc_bhc['Sun']
        end
        business_hours_config.reject!{|k,v| v.nil?}
        obj[:business_hours] = business_hours_config


        dc_ui_config_string = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'UI_CONFIG')
        dc_ui_config_array = dc_ui_config_string.nil? ? Array.new : JSON.parse(dc_ui_config_string)

        ui_options = Hash.new
        ui_options[:abas_gui_embedded] = dc_ui_config_array.include? 'eks'
        ui_options[:abas_gui_2016] = dc_ui_config_array.include? '2016'
        ui_options[:abas_gui] = dc_ui_config_array.include? '2017'
        ui_options[:webui] = dc_ui_config_array.include? 'webui'
        ui_options[:webclient] = dc_ui_config_array.include? 'webclient'
        
        obj[:ui_options] = ui_options

        edp_configurations = Array.new

        dc_edp_config_string = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'EDP_CONFIG')
        dc_edp_configs = JSON.parse(dc_edp_config_string)
        dc_edp_configs.each_with_index do |obj, index| 

            client = Hash.new
            client[:mandant] = obj['id']
            client[:clientdir] = obj['client']

            edp = Hash.new
            edp[:ip] = obj['host']
            edp[:port] = obj['port']

            edpConfig = Hash.new
            edpConfig[:client] = client
            edpConfig[:edp] = edp          
            edpConfig[:display_name] = obj['displayName']
            edpConfig[:dde_server_name] = obj['ddeServer']
            edpConfig[:rest_api_id] = obj['restApiId']

            # credentials have control scope and cannot be retrieved backwards
            # defaultConfig is always the first config in the array. As there is no parameter in the abas-module.yml, we don't need to provide it back.
            edp_configurations.push(edpConfig)
        end

        obj[:edp_configs] = edp_configurations

        obj[:edp_log_enabled] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'EDP_LOG_ENABLED')
        obj[:edp_lock_wait_duration] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'EDP_LOCK_WAIT_DURATION')   
        obj[:edp_end_session_mode] = DockerUtil.readDockerComposeVariable(@dc_file['services']['camunda']['environment'], 'EDP_END_SESSION_MODE')   
        
        return obj
    end

  end

  class Properties
    def initialize(file)
      @file = file
      @raw = ""
      @data = Hash.new
  
      begin
        @raw = File.read(@file)
      rescue StandardError => e
      end
  
      parse_file
    end
  
    def [](key)
      @data[key].strip
    end
  
    private 
  
    def parse_file
      @raw.each_line do |line|
        result = line.partition("=")
        if result[1] == "="
          @data[result[0]] = result[2]
        end
      end
    end
  
  end

  class DockerUtil
    def self.loadComposeFile(path)
        return YAML.load_file(path) 
    end

    def self.readDockerComposeVariable(file, key)
        param = file.select{|env|env.start_with?(key)}.first
        if param.nil?
            return nil
        end
        return param.split("=").last.strip 
    end
  end

end