#!/bin/bash
#------- ABSCHNITT : Modul Identifikation -------------------------------------
#
# Modul Name       : fts-jetty.sh
# .     Funktion   : Jetty starten, oder stoppen
# .     Verwendung : Full text search
# Autor            : uko
# Verantwortlich   : uko
#------- ABSCHNITT : Modul Beschreibung ---------------------------------------
#
# Aufruf mit parameter -start -stop -run
#
#------------------------------------------------------------------------------

#------- ABSCHNITT : Defines, Prozeduren, Typen und Daten ---------------------
name=`basename $0`
pwd=`pwd`
NOBACKUP_HOMEDIR=
PORT=
DIRNAME=$(dirname $(readlink -f $0))

#------------------------------------------------------------------------------
# Exit-Prozedur zum kontrollierten Beenden
#------------------------------------------------------------------------------
myexit()
{
  if test ! -z "$2";  then echo $name: "$2" >&2; fi
  exit $1
}

inside_docker() {
  # source: https://stackoverflow.com/questions/20010199/how-to-determine-if-a-process-runs-inside-lxc-docker
  [ -f /.dockerenv ]
  return $?
}

#------------------------------------------------------------------------------
# Pruefung auf laufenden Jetty
#------------------------------------------------------------------------------
running()
{
  local pidfile="$1"
  if [ -f "$pidfile" ]
  then
    local PID=$(cat "$pidfile" 2>/dev/null) || return 1
    kill -0 "$PID" 2>/dev/null
    local ex=$?
    if ! inside_docker; then
      return $ex
    fi
    if [ $ex -eq 0 ]; then
      # process exists.
      # is it a zombie process? this happens especially
      # in containers with a broken init process (PID 1)
      [ "$(ps -o state= -p $PID)" != "Z" ] && return
    fi
  fi
  rm -f "$1"
  return 1
}

jetty_stop_inside_docker() {
  if [ ! -f "$JETTYPID" ] ; then
    echo "ERROR: no pid found at $JETTYPID"
    exit 1
  fi

  PID=$(cat "$JETTYPID" 2>/dev/null)
  if [ -z "$PID" ] ; then
    echo "ERROR: no pid id found in $JETTYPID"
    exit 1
  fi
  kill "$PID" 2>/dev/null

  TIMEOUT=30
  while running $JETTYPID; do
    if (( TIMEOUT-- == 0 )); then
      kill -KILL "$PID" 2>/dev/null
    fi

    sleep 1
  done
}

#------------------------------------------------------------------------------
# zeigt die Usage an
#------------------------------------------------------------------------------
usage() {
 echo "" >&2
 echo "usage:" >&2
 echo "   * Option -start: start jetty" >&2
 echo "   * Option -stop: stop jetty" >&2
 echo "   * Option -status: jetty status" >&2
 echo "   * Option -port: jetty port" >&2 
 echo "   * Option -?: usage" >&2
 echo "" >&2
 myexit 1
}

#------- ABSCHNITT : Optionen -------------------------------------------------
ACTION=$1
ERP_CLIENT="$pwd"

while [ $# -gt 0 ]
do
  case $1 in
   -start) ACTION=start
	   ;;
   -stop) ACTION=stop
       ;;
   -status) ACTION=status
       ;;
   -run) ACTION=run
       ;;
   -port) shift; PORT=$1
       ;;
   -m) shift
       ERP_CLIENT="$1"
       ;;
   -?) usage
       ;;
    *) usage
       ;;
  esac
  shift
done

source "$DIRNAME/common.sh"

# Jetty directories
JETTY_HOME=$CLOUDDIR/jetty-dist
JETTY_RUN="$JETTY_BASE"

JETTYPID="$JETTY_BASE/jetty.pid"
JETTYTMPDIR="$JETTY_BASE/tmp"

if [ -n "$JAVAPATH" -a -f "$JAVAPATH/bin/java" ] ; then
   JAVA_HOME=$JAVAPATH
   export JAVA_HOME
fi

if [ ! -f $JETTY_HOME/bin/jetty.sh ]; then
    myexit 1 "Error: File $JETTY_HOME/bin/jetty.sh does not exist."
fi

if [ ! -f $JETTY_BASE/start.ini ] ; then
   myexit 1 "Error: JETTY_BASE directory corrupted. $JETTY_BASE/start.ini not found."
fi

#------- ABSCHNITT : Hauptprogramm --------------------------------------------
# main

mkdir -p $JETTYTMPDIR

case "$ACTION" in
  stop)
    if running $JETTYPID
    then
      if ! inside_docker
      then
        JETTY_HOME="$JETTY_HOME" JETTY_RUN="$JETTY_BASE" JETTY_BASE="$JETTY_BASE" $JETTY_HOME/bin/jetty.sh stop || myexit 1 "Error stopping Jetty: $JETTYDIR/bin/jetty.sh stop"
      else
        echo "Jetty is running inside an erp docker container."
        jetty_stop_inside_docker
      fi
    fi
    rm -rf $JETTYTMPDIR/*
    ;;
  start)
    [ -z $PORT ] && myexit "Missing parameter: -port"
    JETTY_HOME="$JETTY_HOME" JETTY_RUN="$JETTY_BASE" JETTY_BASE="$JETTY_BASE" TMPDIR="$JETTYTMPDIR" JETTY_ARGS="jetty.http.port=$PORT" JAVA_OPTIONS="-Xms128M -Xmx1G" $JETTY_HOME/bin/jetty.sh start || myexit 1 "Error starting Jetty: $JETTY_HOME/bin/jetty.sh start"
    ;;
  status)
    if running $JETTYPID
    then
        echo "Jetty is running."
        exit 0
    else
        echo "Jetty is not running."
        exit 3
    fi
    ;;
  run)
    TMPDIR="$JETTYTMPDIR" JETTY_RUN="$JETTY_BASE" JETTY_ARGS="jetty.http.port=$PORT" $JETTY_HOME/bin/jetty.sh run || myexit 1 "Error running Jetty: $JETTYDIR/bin/jetty.sh run"
esac

touch $JETTY_BASE/jetty.port
echo $PORT > $JETTY_BASE/jetty.port || myexit 1 "Error running Jetty: echo $PORT > $JETTY_BASE/jetty.port"

# Ende
myexit 0
