# Common install update steps for keycloak
# $data is from abas-installer 
# $command is one of either 2 values "INSTALL" or "UPDATE". 
# The create_realm script uses this value to decide what to do,
# see flag overwrite_realm for details of behaviour for these
# 2 values.
define keycloak::install_keycloak($data, $command) {

  $module_version               = $data['metadata']['Version']
  # keycloak is internally versioned in abas docker.registry.abas.sh as well
  $product_version              = $data['version']
  $keycloak_abas_image          = "docker.registry.abas.sh/abas-keycloak:${product_version}"

  $config                       = $data['config']
  $install_dir                  = $config['install_dir']
  $bin_dir                      = "${install_dir}/bin"
  $config_dir                   = "${install_dir}/config"
  $certificates_dir             = "${install_dir}/certificates"
  $theme_dir                    = "${install_dir}/theme"
  $startup_scripts_dir          = "${install_dir}/startup_scripts"
  $log_dir                      = "${install_dir}/log"
  $installation_mkdirs          = [ $bin_dir, $config_dir, $certificates_dir, $startup_scripts_dir, $log_dir ]

  $user                         = $config['user']
  $secure_mode                  = $config['secure_mode']
  $http_port                    = $config['http_port']
  $https_port                   = $config['https_port']

  $realm                        = $config['realm']
  $master_realm                 = $config['master_realm']
  $overwrite_realm              = $config['overwrite_realm']
  $service                      = $data['extout']['issuer']
  $master_realm_endpoint        = $data['extras']['master_realm_endpoint']

  $docker_compose_keycloak_service_name          = $data['extras']['docker_compose_keycloak_service_name']
  $docker_compose_keycloak_postgres_service_name = $data['extras']['docker_compose_keycloak_postgres_service_name']

  $management_http_port = $config['management_http_port']
  $keycloak_loglevel = $config['keycloak_loglevel']
  $root_loglevel = $config['root_loglevel']
  $audience = $data['extout']['audience']

  $target_host_ip  = $data['extout']['host_ip']

  $keycloak_admin_credentials   = abas::credentials($config['keycloak_admin_credentials'])
  $keycloak_admin_username      = $keycloak_admin_credentials["username"]
  $keycloak_admin_password      = $keycloak_admin_credentials["password"]

  $keycloak_abas_bin_dir = '/opt/jboss/abas/bin'
  $keycloak_abas_config_dir = '/opt/jboss/abas/config'
  $keycloak_certificates_dir = '/etc/x509/https'
  $webserver_base_url = $config['webserver_base_url']

  $ancestor_source_list = $webserver_base_url

  $project_name = $config['project_name']

  $keycloak_database = $config['keycloak_database']
  $keycloak_db_credentials   = abas::credentials($config['keycloak_db_credentials'])
  $keycloak_db_user = $keycloak_db_credentials['username']
  $keycloak_db_password = $keycloak_db_credentials['password']

  $redirect_uri         = $data['extras']['redirect_uri']

  $abas_bpm_clientid = $data['extras']['abas_bpm_clientid']
  $abas_webclient_clientid = $data['extras']['abas_webclient_clientid']
  $abas_dashboard_clientid = $data['extras']['abas_dashboard_clientid']
  $abas_gui_clientid = $data['extras']['abas_gui_clientid']
  $abas_user_management_service_clientid = $data['extras']['abas_user_management_service_clientid']

  $authentication     = $data['config']['authentication']
  if ($authentication != undef) and ($authentication["registry_credentials"] != undef) {
      $registry_credentials   = abas::credentials($authentication["registry_credentials"])
      $registry_username      = $registry_credentials["username"]
      $registry_password      = $registry_credentials["password"]
  } else {
      $registry_username      = ''
      $registry_password      = ''
  }
  $needed_packages      = [ 'openssl', 'wget']

  if $secure_mode {
    $scheme = 'https'
    $scheme_port = $https_port
  }
  else {
    $scheme = 'http'
    $scheme_port = $http_port
  }

  package { $needed_packages:
    ensure => 'present',
  }
  -> file { $installation_mkdirs:
    ensure => 'directory',
    owner  => $user,
    group  => $user,
    mode   => '0775',
  }
  -> exec {'Create self signed ssl certificates for keycloak https mode':
    command => "openssl req -newkey rsa:4096 -nodes -keyout tls.key -x509 -days 3650 \
                -out tls.crt -subj ${data['extras']['certificate_subject_value']}",
    cwd     => $certificates_dir,
    creates => [ "${certificates_dir}/tls.key", "${certificates_dir}/tls.crt", ],
    unless  => "test -f ${certificates_dir}/tls.key",
    path    => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
  }
  -> file { "${install_dir}/.env":
    ensure  => file,
    content => template('keycloak/.env.erb'),
    owner   => $user,
    mode    => '0400',
  }
  -> file { "${bin_dir}/global_variables.sh":
    ensure  => file,
    content => template('keycloak/global_variables.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/kc_login_utils.sh":
    ensure  => file,
    content => template('keycloak/kc_login_utils.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/create_realm.sh":
    ensure  => file,
    content => template('keycloak/create_realm.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/create_update_client.sh":
    ensure  => file,
    content => template('keycloak/create_update_client.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/setup_user_management_service.sh":
    ensure  => file,
    content => template('keycloak/setup_user_management_service.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/create_update_all_clients.sh":
    ensure  => file,
    content => template('keycloak/create_update_all_clients.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/setup_groups_and_roles.sh":
    ensure  => file,
    content => template('keycloak/setup_groups_and_roles.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/update-realm.sh":
    ensure  => file,
    content => template('keycloak/update-realm.sh.erb'),
    mode    => '0500',
  }
  -> file { "${bin_dir}/update-ssl-required-option.sh":
    ensure  => file,
    content => template('keycloak/update-ssl-required-option.sh.erb'),
    mode    => '0500',
  }
  -> file { "${config_dir}/abas-bpm-client-configuration.json":
    ensure  => file,
    content => template('keycloak/abas-bpm-client-configuration.json.erb'),
  }
  -> file { "${config_dir}/abas-webclient-client-configuration.json":
    ensure  => file,
    content => template('keycloak/abas-webclient-client-configuration.json.erb'),
  }
  -> file { "${config_dir}/abas-dashboard-client-configuration.json":
    ensure  => file,
    content => template('keycloak/abas-dashboard-client-configuration.json.erb'),
  }
  -> file { "${config_dir}/abas-gui-client-configuration.json":
    ensure  => file,
    content => template('keycloak/abas-gui-client-configuration.json.erb'),
  }
  -> file { "${config_dir}/abas-user-management-service.json":
    ensure  => file,
    content => template('keycloak/abas-user-management-service.json.erb'),
  }
  -> file { "${config_dir}/realm-content-security-policy.json":
    ensure  => file,
    content => template('keycloak/realm-content-security-policy.json.erb'),
  }
  -> file { "${install_dir}/docker-compose.yml":
    ensure  => file,
    content => template('keycloak/docker-compose.yml.erb'),
  }
  -> file { "${startup_scripts_dir}/00-enable_access_log.cli":
    source => 'puppet:///modules/keycloak/00-enable_access_log.cli',
  }
  -> file { "${install_dir}/theme.zip":
    source => 'puppet:///modules/keycloak/theme.zip',
  }
  -> exec { 'Delete the abas theme folder if exists':
    command   => 'rm -rf theme',
    cwd       => $install_dir,
    onlyif    => "test -d ${install_dir}/theme",
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Unzipping the abas theme':
    command   => 'unzip theme.zip && rm theme.zip',
    cwd       => $install_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change ownership and permissions of log dir for docker user to read/write/execute':
    command   => "chown -R 1000:1000 ${log_dir} && chmod 775 ${log_dir}",
    cwd       => $install_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change ownership and permissions of certificates dir to read only for user (private key here)':
    command   => 'chown 1000:1000 ./* && chmod 400 ./*',
    cwd       => $certificates_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change ownership docker container user and permissions of config dir to read only for docker user':
    command   => 'chown 1000:1000 ./* && chmod 440 ./*',
    cwd       => $config_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change ownership docker container user and permissions of bin dir to rx only for docker user':
    command   => 'chown 1000:1000 ./* && chmod 550 ./*',
    cwd       => $bin_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change ownership to docker container user and permissions of theme dir files to read only':
    command   => 'find . -type f -exec chown 1000:1000 {} \; && find . -type f -exec chmod 440 {} \;',
    cwd       => $theme_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change ownership to docker container user and permissions of theme dir to read and execute':
    command   => 'find . -type d -exec chown 1000:1000 {} \; && find . -type d -exec chmod 550 {} \;',
    cwd       => $theme_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> file { "${install_dir}/.version":
    ensure  => file,
    content => template('keycloak/.version.erb'),
    mode    => '0440',
    owner   => $user,
  }
  -> file { "${install_dir}/VERSION":
    ensure  => file,
    content => template('keycloak/VERSION.erb'),
    mode    => '0440',
    owner   => $user,
  }
  -> exec { 'DOCKER: Login user to docker.registry.abas.sh':
    onlyif    => ["[ -n '${registry_username}' ] && [ -n '${registry_password}' ]"],
    command   => "docker login docker.registry.abas.sh -u '${registry_username}' -p '${registry_password}'",
    logoutput => true,
    user      => $user,
    path      => '/usr/bin',
  }
  -> file { "${install_dir}/service-defaults.env":
    ensure  => file,
    content => template('keycloak/service-defaults.env.erb'),
    owner   => $user,
  }
  -> file { "${install_dir}/service.sh":
    ensure  => file,
    content => template('keycloak/service.sh.erb'),
    owner   => $user,
    group   => $user,
    mode    => '0500',
  }
  -> exec { 'Start services':
    command => './service.sh --up -- --force-recreate',
    path    => "/usr/bin:/bin:/usr/local/bin:${install_dir}",
    cwd     => $install_dir,
    user    => $user,
  }
  -> exec { 'Sleep wait for server':
    command   => 'sleep 12s',
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Check and wait for server to come online':
    command   => "wget --spider --wait=300 --tries 300 --no-proxy --retry-connrefused -d http://127.0.0.1:${http_port}/auth/realms/${master_realm}",
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { "Creating realm: ${realm}":
    command   => "docker-compose exec -u 1000 -e COMPOSE_INTERACTIVE_NO_CLI=1 -T ${docker_compose_keycloak_service_name} \
                  sh -c '${keycloak_abas_bin_dir}/create_realm.sh'",
    logoutput => true,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    cwd       => $install_dir,
  }
  -> exec { "Update Content Security Header of the realm: ${realm}":
    command   => "docker-compose exec -u 1000 -e COMPOSE_INTERACTIVE_NO_CLI=1 -T ${docker_compose_keycloak_service_name} \
                  sh -c '${keycloak_abas_bin_dir}/update-realm.sh realm-content-security-policy.json'",
    logoutput => true,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    cwd       => $install_dir,
  }
  -> exec { 'Run Load Client scripts':
    command   => "docker-compose exec -u 1000 -e COMPOSE_INTERACTIVE_NO_CLI=1 -T ${docker_compose_keycloak_service_name} \
                  sh -c '${keycloak_abas_bin_dir}/create_update_all_clients.sh'",
    logoutput => true,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    cwd       => $install_dir,
  }
  -> exec { 'Create Groups / Roles / Client Roles and Mappers':
    command   => "docker-compose exec -u 1000 -e COMPOSE_INTERACTIVE_NO_CLI=1 -T ${docker_compose_keycloak_service_name} \
                  sh -c '${keycloak_abas_bin_dir}/setup_groups_and_roles.sh'",
    logoutput => true,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    cwd       => $install_dir,
  }
}
