# abas_license_controller.rb
require 'etc'

# Parameters of scope control don't need to be verified here
#
# check installation drift with this command:
# abas-installer check-module-installations -f <abasfile> -m .
module AbasLicense_controller
  class Main
    def initialize(id_hash)
      @install_dir = id_hash['install_dir']
      if exists?
        @container = DockerService.inspect_container(@install_dir)
      else
        @container = DockerService.nil_container
      end
    end

    def exists?
      return File.file?(version_file)
    end

    def product_version
      return File.read(version_file).strip
    end

    def port
      return @container.port_mapping.fetch('8080/tcp', '')
    end

    def stage
      url = @container.environment.fetch('LICENSE_SERVER_URL', '')
      apex = url[8..-1]
      return '' if apex.nil?
      stage = apex.split('.')[1]
      if stage.nil?
        return ''
      else
        return stage
      end
    end

    def loglevel
      return @container.environment.fetch('LOG_LEVEL', '')
    end

    def user
      uid = File.stat("#{@install_dir}").uid
      return Etc.getpwuid(uid).name
    end

    private

    def version_file
      return File.join(@install_dir, "VERSION")
    end
  end

  class DockerService
    class << self
      def inspect_container(install_dir)
        container_id = container_id(install_dir)

        if container_id.nil? || container_id.empty?
          return nil_container
        else
          return Container.new(environment(container_id), port_mapping(container_id))
        end
      end

      def nil_container
        Container.new({}, {})
      end

      private

      # Obtains the container id of the service.
      # @param install_dir the module's installation directory.
      # @return the container id or nil if it is not started
      def container_id(install_dir)
        container_id = `cd "#{install_dir}" && ./service.sh --status -- -q`
        return nil unless $?.success?
        container_id = container_id.strip
        if container_id.empty?
          return nil
        else
          return container_id
        end
      end

      # Obtains a map of environment variables for a container identified by an id
      def environment(container_id)
        # https://stackoverflow.com/a/39993106/1354318
        raw_environment = `docker inspect -f '{{range $index, $value := .Config.Env}}{{$value}} {{end}}' '#{container_id}'`
        unless $?.success?
          return {}
        else
          return raw_environment.strip.split(' ')
                     .map { |entry| entry.split('=').first(2) }
                     .to_h
        end
      end

      # Obtains a map of port assignments for a container identified by an id. Key is the internal port specification,
      # e.g. 8080/tcp.
      def port_mapping(container_id)
        # https://docs.docker.com/engine/reference/commandline/inspect/#list-all-port-bindings
        raw_mapping = `docker inspect --format='{{range $p, $conf := .NetworkSettings.Ports}} {{$p}}->{{(index $conf 0).HostPort}} {{end}}' '#{container_id}'`
        unless $?.success?
          return {}
        else
          return raw_mapping.strip.split(' ')
                     .map { |entry| entry.split('->').first(2) }
                     .to_h
        end
      end
    end

    class Container
      attr_reader :environment, :port_mapping

      def initialize(environment, port_mapping)
        @environment = environment
        @port_mapping = port_mapping
      end
    end
  end
end