# abas_rest_api.rb
require 'yaml'
require 'uri'

module AbasRest_api
  class Main

    def initialize(id_hash)
        @install_dir = id_hash['install_dir']
        @name = id_hash['name']
        if exists?
          @docker_compose = loadDockerComposeRestApi
          @application_properties = load_properties(File.join(@install_dir, "/application.properties"))
        else
          @docker_compose = {}
          @application_properties = {}
        end
    end

    def product_version
      return File.read("#{@install_dir}/VERSION").strip
    end

    def exists?()
      if ( File.file?(File.join(@install_dir, "/service.sh"))  &&
          File.file?(File.join(@install_dir, "/VERSION")) &&
          File.file?(File.join(@install_dir, "/docker-compose.yml")) &&
          File.file?(File.join(@install_dir, "/application.properties")) &&
          File.file?(File.join(@install_dir, "/service-defaults.env")) )
        return true
      end
      return false
    end

    def user
      if File.exists?("#{@install_dir}")
        uid = File.stat("#{@install_dir}").uid
        return Etc.getpwuid(uid).name
      end
      return "Install directory not found."
    end

    def project_name
      return `COMPOSE_PROJECT_NAME=unknown && source "#{@install_dir}/.env" && echo $COMPOSE_PROJECT_NAME`
    end

    def port
      return @docker_compose.dig('services', 'rest-api', 'ports')[0].split(':')[1]
    end

    def service_ip
      return @docker_compose.dig('services', 'rest-api', 'ports')[0].split(':')[0]
    end

    def jvm_opts
      return readDockerComposeVariable(@docker_compose.dig('services', 'rest-api', 'environment'), 'JAVA_OPTS')
    end

    def access_log_max_history_days
      return readDockerComposeVariable(@docker_compose.dig('services', 'rest-api', 'environment'), 'ACCESS_LOG_MAX_HISTORY_DAYS')
    end

    def access_log_max_total_size
      return readDockerComposeVariable(@docker_compose.dig('services', 'rest-api', 'environment'), 'ACCESS_LOG_MAX_TOTAL_SIZE')
    end

    def enable_web_sockets
      return @application_properties['de.abas.mw.core.edpWebSocketEnabled']
    end

    def detailed_error_messages
      return @application_properties['de.abas.mw.core.deliverDetailedErrorMessages']
    end

    def loglevel_de_abas_mw
      return @application_properties['logging.level.de.abas.mw']
    end

    def loglevel_de_abas_mw_edp
      return @application_properties['logging.level.de.abas.mw.edp']
    end

    def loglevel_root
      return @application_properties['logging.level.root']
    end

    def external_docker_network
      return @docker_compose.dig('networks', 'default', 'external', 'name')
    end

    def clients
      clients = []
      erp_cfg_files = Dir.glob(File.join(@install_dir, "/abasconfig/erp.connection-*.cfg"))
      erp_cfg_files.each do |cfg|
          properties = load_properties(cfg)
          id = properties['id']
          search_config = load_properties(File.join(@install_dir, "/abasconfig/search.service-#{id}.cfg"))
          client = Hash.new
          edp = Hash.new
          uri = URI(properties['uri'])
          edp['hostname'] = uri.host
          edp['port'] = uri.port
          client['edp'] = edp
          client['use_ip'] = uri.host =~ /[\d.]+/ ? true : false
          client['id'] = properties['id']
          client['description'] = properties['description']
          se_string_uri = search_config['uri']
          if se_string_uri != nil
            se_uri = URI(se_string_uri.gsub('{', '%7B').gsub('}', '%7D'))
            client['search_engine_port'] = se_uri.port
          end
          abas_client = Hash.new
          abas_client['mname'] = properties['id']
          abas_client['clientdir'] = uri.path[1..-1]
          abas_client['mandant'] = properties['description']
          client['client'] = abas_client
          client['max_pool_size'] = properties['maxPoolSize']
          client['max_pool_idle'] = properties['maxPoolIdle']
          client['max_pool_wait'] = properties['maxWait']
          client['disable_pool'] = properties['disablePool'] == "true" ? true : false
          client['ws_timeout_minutes'] = properties['workingSetTimeOutInMinutes']
          client['initialize_meta_data_on_startup'] = properties['initializeMetaDataOnStartup']
          client['time_between_two_pool_controls'] = properties['timeBetweenTwoPoolControls']
          client['max_ftext_size'] = properties['maxFTextSize']
          client['metadata_polling_interval_in_seconds'] = properties['metadataPollingIntervalInSeconds']
          client['edp_log_cache_size'] = properties['edpLogCacheSize']
          client['force_edp_client_version'] = properties['forceEdpClientVersion']
          clients.push(client)
      end
      return clients
    end

    private

    def loadDockerComposeRestApi
      begin
          return YAML.load_file(File.join(@install_dir, "/docker-compose.yml"))
      rescue
          return {}
      end
    end

    def load_properties(properties_filename)
      properties = {}
      begin
        File.open(properties_filename, 'r') do |properties_file|
          properties_file.read.each_line do |line|
            line.strip!
            if (line[0] != ?# and line[0] != ?=)
              i = line.index('=')
              if (i)
                properties[line[0..i - 1].strip] = line[i + 1..-1].strip
              else
                properties[line] = ''
              end
            end
          end
        end
      rescue
      end
      properties
    end
    
    def readDockerComposeVariable(file, key)
      return file.select{|env|env.start_with?(key)}.first.split("=").last.strip
    end

  end
end
