# This is the main webserver installation manifest file
define webserver::install_webserver($data) {

  $module_version           = $data['metadata']['Version']
  # webserver is internally versioned in abas docker.registry.abas.sh as well
  $product_version          = $data['version']
  $webserver_abas_image = "docker.registry.abas.sh/abas-nginx:${product_version}"

  # Direct Input Variables defined by the user in abasfile
  $config                   = $data['config']
  $install_dir              = $config['install_dir']
  $hostname                 = $data['extout']['hostname']
  $user                     = $config['user']
  $project_name             = $config['project_name']
  $secure_mode              = $config['secure_mode']
  $debug_mode               = $config['debug_mode']
  $http_port                = $config['http_port']
  $https_port               = $config['https_port']
  $extra_hosts              = $data['hosts']

  # The directories and paths that we will use on the host system where we are installing the module
  $webserver_dir                        = $data['extras']['webserver_dir']
  $webserver_conf_dir                   = $data['extras']['webserver_conf_dir']
  $webserver_confd_dir                  = $data['extras']['webserver_confd_dir']
  $webserver_abas_confd_dir             = $data['extras']['webserver_abas_confd_dir']
  $webserver_abas_confd_dir_name        = $data['extras']['webserver_abas_confd_dir_name']
  $webserver_default_www_conf_file_path = $data['extras']['webserver_default_www_conf_file_path']
  $webserver_conf_file_path             = $data['extras']['webserver_conf_file_path']

  $certificates_dir       = $data['extras']['certificates_dir']
  $webroot_dir            = $data['extras']['webroot_dir']
  $backup_dir             = $data['extras']['backup_dir']

  $docker_compose_service_name = $data['extras']['docker_compose_service_name']

  $webserver_mkdirs       = [ $webserver_conf_dir, $webserver_confd_dir,
                              $webserver_abas_confd_dir, $certificates_dir, $webroot_dir ]

  # The directories and paths that we will use on the docker container running on the host system where 
  # we are installing the module
  $webserver_container_webroot_dir = $data['extras']['webserver_container_webroot_dir']
  $webserver_container_certificates_dir = $data['extras']['webserver_container_certificates_dir']
  $webserver_container_var_log_dir = $data['extras']['webserver_container_var_log_dir']
  $webserver_container_confd_dir = $data['extras']['webserver_container_confd_dir']
  $webserver_container_conf_file_path = $data['extras']['webserver_container_conf_file_path']
  $webserver_container_default_www_conf_file = $data['extras']['webserver_container_default_www_conf_file']
  $webserver_workflow_conf_file_path = $data['extras']['webserver_workflow_conf_file_path']

  # Repo download variables
  $registry_credentials   = abas::credentials($data['config']["registry_credentials"])
  $registry_username      = $registry_credentials["username"]
  $registry_password      = $registry_credentials["password"]
  $repo_base_url          = $config['repo_base_url']

  # Template Variables for Configuration.json
  $meta_json_string = $data['extras']['meta_json_string']
  $clients_json_string = $data['extras']['clients_json_string']
  $api_json_string = $data['extras']['api_json_string']

  # The proxied configuration string for nginx
  $nginx_debug_command = $data['extras']['nginx_debug_command']

  $needed_packages = ['openssl']

  if $config['oauth2'] != undef {
    $on_premise               = $config['oauth2']['type'] == 'keycloak'
    $auth_base_url            = $config['oauth2']['issuer']
  }

  package { $needed_packages:
    ensure => 'present',
  }
  -> exec {'Creating Webserver directory':
    command => "mkdir -p ${webserver_dir}",
    onlyif  => "test ! -d ${webserver_dir}",
    path    => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
  }
  -> file { $webserver_mkdirs:
    ensure => 'directory',
    owner  => $user,
    mode   => '0770',
  }
  -> exec {'create_self_signed_sslcert':
    command => "openssl req -newkey rsa:4096 -nodes -keyout ${hostname}.key -x509 -days 3650 \
                -out ${hostname}.crt -subj ${data['extras']['certificate_subject_value']}",
    cwd     => $certificates_dir,
    unless  => "test -f ${certificates_dir}/${hostname}.key",
    creates => [ "${certificates_dir}/${hostname}.key", "${certificates_dir}/${hostname}.crt", ],
    path    => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
  }
  -> file { $webserver_conf_file_path:
    ensure  => file,
    content => template('webserver/nginx.conf.erb'),
    mode    => '0440',
  }
  -> file { $webserver_default_www_conf_file_path:
    ensure  => file,
    content => template('webserver/nginx.default.conf.erb'),
    mode    => '0440',
  }
  -> file { "${webserver_dir}/docker-compose.yml":
    ensure  => file,
    content => template('webserver/docker-compose.yml.erb'),
    mode    => '0440',
  }
  -> file { "${webserver_dir}/.env":
    ensure  => file,
    content => template('webserver/.env.erb'),
    mode    => '0440',
  }
  -> webserver::download_web_artifacts { 'Download web artifacts to host':
    data    => $data,
  }
  -> file { "${webroot_dir}/configuration.json":
    ensure  => file,
    content => template('webserver/configuration.json.erb')
  }
  -> webserver::configure_freetexteditor { 'Create abas Freetexteditor webserver configuration':
    data    => $data,
  }
  -> webserver::configure_rest_api { 'Create rest-api webserver configuration':
    data    => $data,
  }
  -> webserver::configure_webclient { 'Create webclient webserver configuration':
    data    => $data,
  }
  -> webserver::configure_dashboard { 'Create dashboard webserver configuration':
    data    => $data,
  }
  -> webserver::configure_bpm { 'Create BPM (Reverse Proxy Only) webserver configuration':
    data    => $data,
  }
  -> webserver::configure_user_administration { 'Create user administration configuration':
    data    => $data,
  }
  -> exec { 'Change ownership of install to provided user':
    command   => "chown -hR ${user} ${webserver_dir}",
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change the owner/group of public_html/webroot to owner:root/group:root in nginx container':
    command   => 'chown -hR 0:0 .',
    cwd       => $webroot_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Fix the permissions of public_html/webroot files':
    command   => 'find . -type f -exec chmod 0444 {} \;',
    cwd       => $webroot_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Fix the permissions of public_html/webroot directories':
    command   => 'find . -type d -exec chmod 2555 {} \;',
    cwd       => $webroot_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change the permissions of nginx conf files':
    command   => 'find . -type f -exec chmod 0440 {} \;',
    cwd       => $webserver_conf_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> exec { 'Change the permissions of nginx conf directory':
    command   => 'find . -type d -exec chmod 2550 {} \;',
    cwd       => $webserver_conf_dir,
    path      => '/usr/bin:/usr/sbin:/bin:/usr/local/bin',
    logoutput => 'on_failure',
  }
  -> file { "${webserver_dir}/.version":
    ensure  => file,
    content => template('webserver/.version.erb'),
    mode    => '0440',
    owner   => $user,
  }
  -> file { "${install_dir}/VERSION":
    ensure  => file,
    content => template('webserver/VERSION.erb'),
    mode    => '0440',
    owner   => $user,
  }
  -> exec { 'DOCKER: Login user to docker.registry.abas.sh':
    onlyif    => ["[ -n '${registry_username}' ] && [ -n '${registry_password}' ]"],
    command   => "docker login docker.registry.abas.sh -u '${registry_username}' -p '${registry_password}'",
    logoutput => true,
    user      => $user,
    path      => '/usr/bin',
  }
  -> file { "${webserver_dir}/service-defaults.env":
    ensure  => file,
    content => template('webserver/service-defaults.env.erb'),
    owner   => $user,
  }
  -> file { "${webserver_dir}/service.sh":
    ensure  => file,
    content => template('webserver/service.sh.erb'),
    owner   => $user,
    mode    => '0500',
  }
  -> exec { 'Start the Nginx Webserver':
    command => "./service.sh --up -- --force-recreate",
    path    => "/usr/bin:/bin:/usr/local/bin:${webserver_dir}",
    cwd     => $webserver_dir,
    user    => $user,
    logoutput => true,
  }
}
